export interface Result<T> {
    amountDone: number;
    amountStarted: number;
    amountResolved: number;
    amountRejected: number;
    rejectedIndexes: number[];
    resolvedIndexes: number[];
    taskResults: T[];
}
export declare type Task<T> = () => Promise<T>;
export declare type Tasks<T> = Array<Task<T>>;
export declare type nextTaskCheck = <T>(status: Result<T>, tasks: Tasks<T>) => Promise<boolean>;
/**
 * Raw throttle function, which can return extra meta data.
 * @param tasks array[] of functions
 * @param maxInProgress integer with the max amount of tasks to be run in parallel
 * @param failFast boolean if true, do fail-fast behaviour (see Promise.all() documentation)
 * @param progressCallback function which can be used to see the progress
 * @param nextCheck
 * @returns {Promise}
 */
export declare function raw<T>(tasks: Tasks<T>, maxInProgress?: number, failFast?: boolean, progressCallback?: (result: Result<T>) => void, nextCheck?: nextTaskCheck): Promise<Result<T>>;
/**
 * Simply run all the promises after each other, so in synchronous manner
 * @param tasks required array of tasks to be executed
 * @param failFast optional boolean if we directly reject on a single error defaults to true
 * @param progressCallback optional function to be run to get status updates
 * @param nextCheck function which should return a promise, when resolved the next task will spawn
 */
export declare function sync<T>(tasks: Tasks<T>, failFast?: boolean, progressCallback?: () => void, nextCheck?: nextTaskCheck): Promise<T[]>;
/**
 * Exposes the same behaviour as Promise.All(), but throttled!
 * @param tasks required array of tasks to be executed
 * @param maxInProgress optional integer max amount of parallel tasks to be run defaults to 5
 * @param failFast optional boolean if we directly reject on a single error defaults to true
 * @param progressCallback optional function to be run to get status updates
 * @param nextCheck function which should return a promise, when resolved the next task will spawn
 */
export declare function all<T>(tasks: Tasks<T>, maxInProgress?: number, failFast?: boolean, progressCallback?: () => void, nextCheck?: nextTaskCheck): Promise<T[]>;
